from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .global_secret_item_tags import GlobalSecretItem_tags

@dataclass
class GlobalSecretItem(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The createdAt property
    created_at: Optional[datetime.datetime] = None
    # The description property
    description: Optional[str] = None
    # The ignoreInScan property
    ignore_in_scan: Optional[bool] = None
    # The key property
    key: Optional[str] = None
    # The tags property
    tags: Optional[GlobalSecretItem_tags] = None
    # The type property
    type: Optional[str] = None
    # The updatedAt property
    updated_at: Optional[datetime.datetime] = None
    # The value property
    value: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> GlobalSecretItem:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: GlobalSecretItem
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return GlobalSecretItem()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .global_secret_item_tags import GlobalSecretItem_tags

        from .global_secret_item_tags import GlobalSecretItem_tags

        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "ignoreInScan": lambda n : setattr(self, 'ignore_in_scan', n.get_bool_value()),
            "key": lambda n : setattr(self, 'key', n.get_str_value()),
            "tags": lambda n : setattr(self, 'tags', n.get_object_value(GlobalSecretItem_tags)),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "value": lambda n : setattr(self, 'value', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("description", self.description)
        writer.write_bool_value("ignoreInScan", self.ignore_in_scan)
        writer.write_str_value("key", self.key)
        writer.write_object_value("tags", self.tags)
        writer.write_str_value("type", self.type)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("value", self.value)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class BillingStatementResponse(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The finalizedAt property
    finalized_at: Optional[datetime.datetime] = None
    # The freeQuota property
    free_quota: Optional[int] = None
    # The id property
    id: Optional[UUID] = None
    # The isFinalized property
    is_finalized: Optional[bool] = None
    # The lastUpdatedAt property
    last_updated_at: Optional[datetime.datetime] = None
    # The month property
    month: Optional[str] = None
    # The overageCost property
    overage_cost: Optional[float] = None
    # The overageRate property
    overage_rate: Optional[float] = None
    # The overageRequests property
    overage_requests: Optional[int] = None
    # The plan property
    plan: Optional[str] = None
    # The requestsUsed property
    requests_used: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> BillingStatementResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: BillingStatementResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return BillingStatementResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "finalizedAt": lambda n : setattr(self, 'finalized_at', n.get_datetime_value()),
            "freeQuota": lambda n : setattr(self, 'free_quota', n.get_int_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isFinalized": lambda n : setattr(self, 'is_finalized', n.get_bool_value()),
            "lastUpdatedAt": lambda n : setattr(self, 'last_updated_at', n.get_datetime_value()),
            "month": lambda n : setattr(self, 'month', n.get_str_value()),
            "overageCost": lambda n : setattr(self, 'overage_cost', n.get_float_value()),
            "overageRate": lambda n : setattr(self, 'overage_rate', n.get_float_value()),
            "overageRequests": lambda n : setattr(self, 'overage_requests', n.get_int_value()),
            "plan": lambda n : setattr(self, 'plan', n.get_str_value()),
            "requestsUsed": lambda n : setattr(self, 'requests_used', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("finalizedAt", self.finalized_at)
        writer.write_int_value("freeQuota", self.free_quota)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isFinalized", self.is_finalized)
        writer.write_datetime_value("lastUpdatedAt", self.last_updated_at)
        writer.write_str_value("month", self.month)
        writer.write_float_value("overageCost", self.overage_cost)
        writer.write_float_value("overageRate", self.overage_rate)
        writer.write_int_value("overageRequests", self.overage_requests)
        writer.write_str_value("plan", self.plan)
        writer.write_int_value("requestsUsed", self.requests_used)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .claim_operator import ClaimOperator

@dataclass
class ClaimRule(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The claim property
    claim: Optional[str] = None
    # The operator property
    operator: Optional[ClaimOperator] = None
    # The value property
    value: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ClaimRule:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ClaimRule
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ClaimRule()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .claim_operator import ClaimOperator

        from .claim_operator import ClaimOperator

        fields: dict[str, Callable[[Any], None]] = {
            "claim": lambda n : setattr(self, 'claim', n.get_str_value()),
            "operator": lambda n : setattr(self, 'operator', n.get_enum_value(ClaimOperator)),
            "value": lambda n : setattr(self, 'value', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("claim", self.claim)
        writer.write_enum_value("operator", self.operator)
        writer.write_str_value("value", self.value)
        writer.write_additional_data_value(self.additional_data)
    




"""HmacAuthProvider — Kiota AuthenticationProvider that signs requests with HMAC-SHA256."""

from __future__ import annotations

import hashlib
import hmac as hmac_mod
import os
from datetime import datetime, timezone
from urllib.parse import urlparse, parse_qsl, urlencode


class HmacAuthProvider:
    """Kiota AuthenticationProvider that adds HMAC-SHA256 signing headers to every request."""

    def __init__(self, api_key: str, bella_client: str = "bella-python-sdk", app_client: str | None = None) -> None:
        parts = api_key.split("-", 2)
        if len(parts) != 3 or parts[0] != "bax":
            raise ValueError("api_key must be in format bax-{keyId}-{signingSecret}")
        self.key_id = parts[1]
        self.signing_secret = bytes.fromhex(parts[2])
        self.bella_client = bella_client
        self.app_client = app_client or os.environ.get("BELLA_BAXTER_APP_CLIENT")

    async def authenticate_request(self, request, additional_auth_context=None) -> None:
        method = request.http_method.value if hasattr(request.http_method, "value") else str(request.http_method).upper()
        url = str(request.url)
        parsed = urlparse(url)
        path = parsed.path
        query = urlencode(sorted(parse_qsl(parsed.query or "")))
        body = b""
        if request.content:
            body = request.content if isinstance(request.content, bytes) else request.content.read()
        timestamp = datetime.now(timezone.utc).strftime("%Y-%m-%dT%H:%M:%SZ")
        body_hash = hashlib.sha256(body).hexdigest()
        string_to_sign = f"{method}\n{path}\n{query}\n{timestamp}\n{body_hash}"
        sig = hmac_mod.new(self.signing_secret, string_to_sign.encode(), hashlib.sha256).hexdigest()
        request.headers.try_add("X-Bella-Key-Id", self.key_id)
        request.headers.try_add("X-Bella-Timestamp", timestamp)
        request.headers.try_add("X-Bella-Signature", sig)
        request.headers.try_add("X-Bella-Client", self.bella_client)
        if self.app_client:
            request.headers.try_add("X-App-Client", self.app_client)



from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ApiKeyResponse(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The allowedScopes property
    allowed_scopes: Optional[list[str]] = None
    # The clientName property
    client_name: Optional[str] = None
    # The createdAt property
    created_at: Optional[datetime.datetime] = None
    # The description property
    description: Optional[str] = None
    # The environmentId property
    environment_id: Optional[str] = None
    # The expiresAt property
    expires_at: Optional[datetime.datetime] = None
    # The id property
    id: Optional[str] = None
    # The isActive property
    is_active: Optional[bool] = None
    # The isExpired property
    is_expired: Optional[bool] = None
    # The isValid property
    is_valid: Optional[bool] = None
    # The keyPrefix property
    key_prefix: Optional[str] = None
    # The lastUsedAt property
    last_used_at: Optional[datetime.datetime] = None
    # The projectId property
    project_id: Optional[str] = None
    # The revokedAt property
    revoked_at: Optional[datetime.datetime] = None
    # The role property
    role: Optional[str] = None
    # The usageCount property
    usage_count: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ApiKeyResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ApiKeyResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ApiKeyResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "allowedScopes": lambda n : setattr(self, 'allowed_scopes', n.get_collection_of_primitive_values(str)),
            "clientName": lambda n : setattr(self, 'client_name', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "environmentId": lambda n : setattr(self, 'environment_id', n.get_str_value()),
            "expiresAt": lambda n : setattr(self, 'expires_at', n.get_datetime_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "isActive": lambda n : setattr(self, 'is_active', n.get_bool_value()),
            "isExpired": lambda n : setattr(self, 'is_expired', n.get_bool_value()),
            "isValid": lambda n : setattr(self, 'is_valid', n.get_bool_value()),
            "keyPrefix": lambda n : setattr(self, 'key_prefix', n.get_str_value()),
            "lastUsedAt": lambda n : setattr(self, 'last_used_at', n.get_datetime_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_str_value()),
            "revokedAt": lambda n : setattr(self, 'revoked_at', n.get_datetime_value()),
            "role": lambda n : setattr(self, 'role', n.get_str_value()),
            "usageCount": lambda n : setattr(self, 'usage_count', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("allowedScopes", self.allowed_scopes)
        writer.write_str_value("clientName", self.client_name)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("description", self.description)
        writer.write_str_value("environmentId", self.environment_id)
        writer.write_datetime_value("expiresAt", self.expires_at)
        writer.write_str_value("id", self.id)
        writer.write_bool_value("isActive", self.is_active)
        writer.write_bool_value("isExpired", self.is_expired)
        writer.write_bool_value("isValid", self.is_valid)
        writer.write_str_value("keyPrefix", self.key_prefix)
        writer.write_datetime_value("lastUsedAt", self.last_used_at)
        writer.write_str_value("projectId", self.project_id)
        writer.write_datetime_value("revokedAt", self.revoked_at)
        writer.write_str_value("role", self.role)
        writer.write_int_value("usageCount", self.usage_count)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class E2EEncryptedPayload(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The algorithm property
    algorithm: Optional[str] = None
    # The ciphertext property
    ciphertext: Optional[str] = None
    # The encrypted property
    encrypted: Optional[bool] = None
    # The nonce property
    nonce: Optional[str] = None
    # The serverPublicKey property
    server_public_key: Optional[str] = None
    # The tag property
    tag: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> E2EEncryptedPayload:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: E2EEncryptedPayload
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return E2EEncryptedPayload()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "algorithm": lambda n : setattr(self, 'algorithm', n.get_str_value()),
            "ciphertext": lambda n : setattr(self, 'ciphertext', n.get_str_value()),
            "encrypted": lambda n : setattr(self, 'encrypted', n.get_bool_value()),
            "nonce": lambda n : setattr(self, 'nonce', n.get_str_value()),
            "serverPublicKey": lambda n : setattr(self, 'server_public_key', n.get_str_value()),
            "tag": lambda n : setattr(self, 'tag', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("algorithm", self.algorithm)
        writer.write_str_value("ciphertext", self.ciphertext)
        writer.write_bool_value("encrypted", self.encrypted)
        writer.write_str_value("nonce", self.nonce)
        writer.write_str_value("serverPublicKey", self.server_public_key)
        writer.write_str_value("tag", self.tag)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .claim_rule import ClaimRule

@dataclass
class UpdateTrustDomainCommand(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The claimRules property
    claim_rules: Optional[list[ClaimRule]] = None
    # The description property
    description: Optional[str] = None
    # The isEnabled property
    is_enabled: Optional[bool] = None
    # The issuedTokenTtlMinutes property
    issued_token_ttl_minutes: Optional[int] = None
    # The name property
    name: Optional[str] = None
    # The oidcIssuerUrl property
    oidc_issuer_url: Optional[str] = None
    # The oidcJwksUri property
    oidc_jwks_uri: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> UpdateTrustDomainCommand:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: UpdateTrustDomainCommand
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return UpdateTrustDomainCommand()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .claim_rule import ClaimRule

        from .claim_rule import ClaimRule

        fields: dict[str, Callable[[Any], None]] = {
            "claimRules": lambda n : setattr(self, 'claim_rules', n.get_collection_of_object_values(ClaimRule)),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "isEnabled": lambda n : setattr(self, 'is_enabled', n.get_bool_value()),
            "issuedTokenTtlMinutes": lambda n : setattr(self, 'issued_token_ttl_minutes', n.get_int_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "oidcIssuerUrl": lambda n : setattr(self, 'oidc_issuer_url', n.get_str_value()),
            "oidcJwksUri": lambda n : setattr(self, 'oidc_jwks_uri', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("claimRules", self.claim_rules)
        writer.write_str_value("description", self.description)
        writer.write_bool_value("isEnabled", self.is_enabled)
        writer.write_int_value("issuedTokenTtlMinutes", self.issued_token_ttl_minutes)
        writer.write_str_value("name", self.name)
        writer.write_str_value("oidcIssuerUrl", self.oidc_issuer_url)
        writer.write_str_value("oidcJwksUri", self.oidc_jwks_uri)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import ComposedTypeWrapper, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .group_operation_response_group_member1 import GroupOperationResponse_groupMember1
    from .group_response import GroupResponse

@dataclass
class GroupOperationResponse_group(ComposedTypeWrapper, Parsable):
    """
    Composed type wrapper for classes GroupOperationResponse_groupMember1, GroupResponse
    """
    # Composed type representation for type GroupOperationResponse_groupMember1
    group_operation_response_group_member1: Optional[GroupOperationResponse_groupMember1] = None
    # Composed type representation for type GroupResponse
    group_response: Optional[GroupResponse] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> GroupOperationResponse_group:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: GroupOperationResponse_group
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        try:
            child_node = parse_node.get_child_node("")
            mapping_value = child_node.get_str_value() if child_node else None
        except AttributeError:
            mapping_value = None
        result = GroupOperationResponse_group()
        if mapping_value and mapping_value.casefold() == "".casefold():
            from .group_operation_response_group_member1 import GroupOperationResponse_groupMember1

            result.group_operation_response_group_member1 = GroupOperationResponse_groupMember1()
        elif mapping_value and mapping_value.casefold() == "GroupResponse".casefold():
            from .group_response import GroupResponse

            result.group_response = GroupResponse()
        return result
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .group_operation_response_group_member1 import GroupOperationResponse_groupMember1
        from .group_response import GroupResponse

        if self.group_operation_response_group_member1:
            return self.group_operation_response_group_member1.get_field_deserializers()
        if self.group_response:
            return self.group_response.get_field_deserializers()
        return {}
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        if self.group_operation_response_group_member1:
            writer.write_object_value(None, self.group_operation_response_group_member1)
        elif self.group_response:
            writer.write_object_value(None, self.group_response)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .diagnostics.diagnostics_request_builder import DiagnosticsRequestBuilder
    from .tenants.tenants_request_builder import TenantsRequestBuilder
    from .users.users_request_builder import UsersRequestBuilder
    from .v1.v1_request_builder import V1RequestBuilder
    from .version.version_request_builder import VersionRequestBuilder

class ApiRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /api
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ApiRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/api", path_parameters)
    
    @property
    def diagnostics(self) -> DiagnosticsRequestBuilder:
        """
        The diagnostics property
        """
        from .diagnostics.diagnostics_request_builder import DiagnosticsRequestBuilder

        return DiagnosticsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def tenants(self) -> TenantsRequestBuilder:
        """
        The tenants property
        """
        from .tenants.tenants_request_builder import TenantsRequestBuilder

        return TenantsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def users(self) -> UsersRequestBuilder:
        """
        The users property
        """
        from .users.users_request_builder import UsersRequestBuilder

        return UsersRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def v1(self) -> V1RequestBuilder:
        """
        The v1 property
        """
        from .v1.v1_request_builder import V1RequestBuilder

        return V1RequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def version(self) -> VersionRequestBuilder:
        """
        The version property
        """
        from .version.version_request_builder import VersionRequestBuilder

        return VersionRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .environment_export_info import EnvironmentExportInfo
    from .project_export_info import ProjectExportInfo
    from .provider_export_info import ProviderExportInfo

@dataclass
class ProjectExportBundle(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The environments property
    environments: Optional[list[EnvironmentExportInfo]] = None
    # The exportVersion property
    export_version: Optional[str] = None
    # The exportedAt property
    exported_at: Optional[datetime.datetime] = None
    # The project property
    project: Optional[ProjectExportInfo] = None
    # The providers property
    providers: Optional[list[ProviderExportInfo]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ProjectExportBundle:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ProjectExportBundle
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ProjectExportBundle()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .environment_export_info import EnvironmentExportInfo
        from .project_export_info import ProjectExportInfo
        from .provider_export_info import ProviderExportInfo

        from .environment_export_info import EnvironmentExportInfo
        from .project_export_info import ProjectExportInfo
        from .provider_export_info import ProviderExportInfo

        fields: dict[str, Callable[[Any], None]] = {
            "environments": lambda n : setattr(self, 'environments', n.get_collection_of_object_values(EnvironmentExportInfo)),
            "exportVersion": lambda n : setattr(self, 'export_version', n.get_str_value()),
            "exportedAt": lambda n : setattr(self, 'exported_at', n.get_datetime_value()),
            "project": lambda n : setattr(self, 'project', n.get_object_value(ProjectExportInfo)),
            "providers": lambda n : setattr(self, 'providers', n.get_collection_of_object_values(ProviderExportInfo)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("environments", self.environments)
        writer.write_str_value("exportVersion", self.export_version)
        writer.write_datetime_value("exportedAt", self.exported_at)
        writer.write_object_value("project", self.project)
        writer.write_collection_of_object_values("providers", self.providers)
        writer.write_additional_data_value(self.additional_data)
    




"""E2EETransport — httpx transport wrapper that adds E2EE to GET /secrets requests."""

from __future__ import annotations

import json

import httpx

from .e2ee import E2EKeyPair, maybe_decrypt, maybe_decrypt_raw


def _add_e2ee_header(request: httpx.Request, public_key_b64: str) -> httpx.Request:
    """Return a new request with X-E2E-Public-Key header added."""
    headers = dict(request.headers)
    headers["X-E2E-Public-Key"] = public_key_b64
    return httpx.Request(
        method=request.method,
        url=request.url,
        headers=headers,
        content=request.content,
    )


def _decrypt_response(response: httpx.Response, e2ee: E2EKeyPair, raw_content: bytes) -> httpx.Response:
    """Decrypt the E2EE-encrypted response body and return a new plain response."""
    import json as _json
    data = _json.loads(raw_content)
    if data.get("encrypted"):
        decrypted = maybe_decrypt_raw(data, e2ee)
        if "secrets" in decrypted and isinstance(decrypted.get("secrets"), dict):
            new_body = _json.dumps(decrypted).encode()
        else:
            secrets = maybe_decrypt(data, e2ee)
            new_body = _json.dumps({"secrets": secrets, "version": 0, "environmentSlug": "", "environmentName": "", "lastModified": ""}).encode()
        return httpx.Response(
            status_code=response.status_code,
            headers=response.headers,
            content=new_body,
        )
    return response


class E2EETransport(httpx.BaseTransport):
    """
    Synchronous httpx transport that transparently handles E2EE for GET /secrets requests.

    On outbound: adds X-E2E-Public-Key header so the server encrypts the response.
    On inbound:  decrypts the encrypted payload and reconstructs a normal JSON response.
    """

    def __init__(self, wrapped: httpx.BaseTransport) -> None:
        self._wrapped = wrapped
        self._e2ee = E2EKeyPair()

    def handle_request(self, request: httpx.Request) -> httpx.Response:
        is_secrets = request.url.path.rstrip("/").endswith("/secrets") and request.method == "GET"

        if is_secrets:
            request = _add_e2ee_header(request, self._e2ee.public_key_b64)

        response = self._wrapped.handle_request(request)

        if is_secrets and response.is_success:
            response.read()
            response = _decrypt_response(response, self._e2ee, response.content)

        return response


class AsyncE2EETransport(httpx.AsyncBaseTransport):
    """
    Async httpx transport that transparently handles E2EE for GET /secrets requests.

    Used by BaxterClient when building the AsyncClient for the Kiota adapter.
    """

    def __init__(self, wrapped: httpx.AsyncBaseTransport) -> None:
        self._wrapped = wrapped
        self._e2ee = E2EKeyPair()

    async def handle_async_request(self, request: httpx.Request) -> httpx.Response:
        is_secrets = request.url.path.rstrip("/").endswith("/secrets") and request.method == "GET"

        if is_secrets:
            request = _add_e2ee_header(request, self._e2ee.public_key_b64)

        response = await self._wrapped.handle_async_request(request)

        if is_secrets and response.is_success:
            await response.aread()
            response = _decrypt_response(response, self._e2ee, response.content)

        return response



from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from ...models.command import Command
    from ...models.create_tenant_response import CreateTenantResponse
    from ...models.list_tenants_response import ListTenantsResponse
    from .invites.invites_request_builder import InvitesRequestBuilder
    from .item.identifier_item_request_builder import IdentifierItemRequestBuilder
    from .my_tenants.my_tenants_request_builder import MyTenantsRequestBuilder
    from .new.new_request_builder import NewRequestBuilder

class TenantsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /api/tenants
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new TenantsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/api/tenants{?page*,pageSize*}", path_parameters)
    
    def by_identifier_id(self,identifier_id: str) -> IdentifierItemRequestBuilder:
        """
        Gets an item from the bella_baxter.generated.api.tenants.item collection
        param identifier_id: Unique identifier of the item
        Returns: IdentifierItemRequestBuilder
        """
        if identifier_id is None:
            raise TypeError("identifier_id cannot be null.")
        from .item.identifier_item_request_builder import IdentifierItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["identifier%2Did"] = identifier_id
        return IdentifierItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[TenantsRequestBuilderGetQueryParameters]] = None) -> Optional[list[ListTenantsResponse]]:
        """
        GET_api_tenants
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[list[ListTenantsResponse]]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ...models.list_tenants_response import ListTenantsResponse

        return await self.request_adapter.send_collection_async(request_info, ListTenantsResponse, None)
    
    async def post(self,body: Command, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[CreateTenantResponse]:
        """
        POST_api_tenants
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[CreateTenantResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ...models.create_tenant_response import CreateTenantResponse

        return await self.request_adapter.send_async(request_info, CreateTenantResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[TenantsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        GET_api_tenants
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: Command, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        POST_api_tenants
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> TenantsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: TenantsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return TenantsRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def invites(self) -> InvitesRequestBuilder:
        """
        The invites property
        """
        from .invites.invites_request_builder import InvitesRequestBuilder

        return InvitesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def my_tenants(self) -> MyTenantsRequestBuilder:
        """
        The myTenants property
        """
        from .my_tenants.my_tenants_request_builder import MyTenantsRequestBuilder

        return MyTenantsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def new(self) -> NewRequestBuilder:
        """
        The new property
        """
        from .new.new_request_builder import NewRequestBuilder

        return NewRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class TenantsRequestBuilderGetQueryParameters():
        """
        GET_api_tenants
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "page_size":
                return "pageSize"
            if original_name == "page":
                return "page"
            return original_name
        
        page: Optional[int] = None

        page_size: Optional[int] = None

    
    @dataclass
    class TenantsRequestBuilderGetRequestConfiguration(RequestConfiguration[TenantsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class TenantsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .security_insight_type import SecurityInsightType
    from .security_risk_level import SecurityRiskLevel

@dataclass
class SecurityInsight(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The detail property
    detail: Optional[str] = None
    # The matchedPattern property
    matched_pattern: Optional[str] = None
    # The recommendation property
    recommendation: Optional[str] = None
    # The secretKey property
    secret_key: Optional[str] = None
    # The severity property
    severity: Optional[SecurityRiskLevel] = None
    # The type property
    type: Optional[SecurityInsightType] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SecurityInsight:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SecurityInsight
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SecurityInsight()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .security_insight_type import SecurityInsightType
        from .security_risk_level import SecurityRiskLevel

        from .security_insight_type import SecurityInsightType
        from .security_risk_level import SecurityRiskLevel

        fields: dict[str, Callable[[Any], None]] = {
            "detail": lambda n : setattr(self, 'detail', n.get_str_value()),
            "matchedPattern": lambda n : setattr(self, 'matched_pattern', n.get_str_value()),
            "recommendation": lambda n : setattr(self, 'recommendation', n.get_str_value()),
            "secretKey": lambda n : setattr(self, 'secret_key', n.get_str_value()),
            "severity": lambda n : setattr(self, 'severity', n.get_enum_value(SecurityRiskLevel)),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(SecurityInsightType)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("detail", self.detail)
        writer.write_str_value("matchedPattern", self.matched_pattern)
        writer.write_str_value("recommendation", self.recommendation)
        writer.write_str_value("secretKey", self.secret_key)
        writer.write_enum_value("severity", self.severity)
        writer.write_enum_value("type", self.type)
        writer.write_additional_data_value(self.additional_data)
    

